@@ -15,8 +15,11 @@ Unless required by applicable law or agreed to in writing, software distributed
 
 from flask import abort, request, send_file, send_from_directory, render_template, render_template_string, \
     url_for
+import werkzeug.exceptions as exceptions
 from .app import App
 from .api import Api
 from .problem import problem
 from .decorators.produces import NoContent
-import werkzeug.exceptions as exceptions
+from .resolver import Resolution, Resolver, RestyResolver
+
+__version__ = '1.0.81'
