@@ -13,19 +13,20 @@ Unless required by applicable law or agreed to in writing, software distributed
 
 import logging
 import pathlib
-
 import flask
 import werkzeug.exceptions
-
 from .problem import problem
 from .api import Api
+from connexion.resolver import Resolver
 
 logger = logging.getLogger('connexion.app')
 
 
-class App:
-    def __init__(self, import_name, port=None, specification_dir='', server=None, arguments=None, debug=False,
-                 swagger_ui=True, swagger_path=None, swagger_url=None):
+class App(object):
+    def __init__(self, import_name, port=None, specification_dir='',
+                 server=None, arguments=None, auth_all_paths=False,
+                 debug=False, swagger_json=True, swagger_ui=True, swagger_path=None,
+                 swagger_url=None):
         """
         :param import_name: the name of the application package
         :type import_name: str
@@ -37,8 +38,12 @@ class App:
         :type server: str | None
         :param arguments: arguments to replace on the specification
         :type arguments: dict | None
+        :param auth_all_paths: whether to authenticate not defined paths
+        :type auth_all_paths: bool
         :param debug: include debugging information
         :type debug: bool
+        :param swagger_json: whether to include swagger json or not
+        :type swagger_json: bool
         :param swagger_ui: whether to include swagger ui or not
         :type swagger_ui: bool
         :param swagger_path: path to swagger-ui directory
@@ -67,10 +72,13 @@ class App:
         self.port = port
         self.server = server or 'flask'
         self.debug = debug
+        self.import_name = import_name
         self.arguments = arguments or {}
+        self.swagger_json = swagger_json
         self.swagger_ui = swagger_ui
         self.swagger_path = swagger_path
         self.swagger_url = swagger_url
+        self.auth_all_paths = auth_all_paths
 
     @staticmethod
     def common_error_handler(exception):
@@ -81,8 +89,9 @@ class App:
             exception = werkzeug.exceptions.InternalServerError()
         return problem(title=exception.name, detail=exception.description, status=exception.code)
 
-    def add_api(self, swagger_file, base_path=None, arguments=None, swagger_ui=None, swagger_path=None,
-                swagger_url=None, validate_responses=False):
+    def add_api(self, swagger_file, base_path=None, arguments=None, auth_all_paths=None, swagger_json=None,
+                swagger_ui=None, swagger_path=None, swagger_url=None, validate_responses=False,
+                resolver=Resolver()):
         """
         Adds an API to the application based on a swagger file
 
@@ -92,6 +101,10 @@ class App:
         :type base_path: str | None
         :param arguments: api version specific arguments to replace on the specification
         :type arguments: dict | None
+        :param auth_all_paths: whether to authenticate not defined paths
+        :type auth_all_paths: bool
+        :param swagger_json: whether to include swagger json or not
+        :type swagger_json: bool
         :param swagger_ui: whether to include swagger ui or not
         :type swagger_ui: bool
         :param swagger_path: path to swagger-ui directory
@@ -100,18 +113,32 @@ class App:
         :type swagger_url: string | None
         :param validate_responses: True enables validation. Validation errors generate HTTP 500 responses.
         :type validate_responses: bool
+        :param resolver: Operation resolver.
+        :type resolver: Resolver | types.FunctionType
         :rtype: Api
         """
+        resolver = Resolver(resolver) if hasattr(resolver, '__call__') else resolver
+
+        swagger_json = swagger_json if swagger_json is not None else self.swagger_json
         swagger_ui = swagger_ui if swagger_ui is not None else self.swagger_ui
         swagger_path = swagger_path if swagger_path is not None else self.swagger_path
         swagger_url = swagger_url if swagger_url is not None else self.swagger_url
+        auth_all_paths = auth_all_paths if auth_all_paths is not None else self.auth_all_paths
         logger.debug('Adding API: %s', swagger_file)
         # TODO test if base_url starts with an / (if not none)
         arguments = arguments or dict()
         arguments = dict(self.arguments, **arguments)  # copy global arguments and update with api specfic
         yaml_path = self.specification_dir / swagger_file
-        api = Api(yaml_path, base_path, arguments,
-                  swagger_ui, swagger_path, swagger_url, validate_responses)
+        api = Api(swagger_yaml_path=yaml_path,
+                  base_url=base_path, arguments=arguments,
+                  swagger_json=swagger_json,
+                  swagger_ui=swagger_ui,
+                  swagger_path=swagger_path,
+                  swagger_url=swagger_url,
+                  resolver=resolver,
+                  validate_responses=validate_responses,
+                  auth_all_paths=auth_all_paths,
+                  debug=self.debug)
         self.app.register_blueprint(api.blueprint)
         return api
 
@@ -227,7 +254,7 @@ class App:
             wsgi_container = tornado.wsgi.WSGIContainer(self.app)
             http_server = tornado.httpserver.HTTPServer(wsgi_container, **options)
             http_server.listen(self.port)
-            logger.info('Listening on port %s..', port=self.port)
+            logger.info('Listening on port %s..', self.port)
             tornado.ioloop.IOLoop.instance().start()
         elif self.server == 'gevent':
             try:
