@@ -1,4 +1,3 @@
-
 import functools
 import os
 import time
@@ -7,12 +6,13 @@ from connexion.decorators.produces import BaseSerializer
 
 try:
     import uwsgi_metrics
-    HAS_UWSGI_METRICS = True
+    HAS_UWSGI_METRICS = True  # pragma: no cover
 except ImportError:
+    uwsgi_metrics = None
     HAS_UWSGI_METRICS = False
 
 
-class UWSGIMetricsCollector:
+class UWSGIMetricsCollector(object):
     def __init__(self, path, method):
         self.path = path
         self.method = method
