@@ -11,28 +11,100 @@ Unless required by applicable law or agreed to in writing, software distributed
  language governing permissions and limitations under the License.
 """
 
+from copy import deepcopy
 import functools
 import logging
-import os
+
+from jsonschema import ValidationError
+
+from .decorators import validation
+from .decorators.metrics import UWSGIMetricsCollector
 from .decorators.parameter import parameter_to_arg
 from .decorators.produces import BaseSerializer, Produces, Jsonifier
-from .decorators.security import security_passthrough, verify_oauth
-from .decorators.validation import RequestBodyValidator, ParameterValidator
-from .decorators.metrics import UWSGIMetricsCollector
-from.decorators.response import ResponseValidator
+from .decorators.response import ResponseValidator
+from .decorators.security import security_passthrough, verify_oauth, get_tokeninfo_url
+from .decorators.validation import RequestBodyValidator, ParameterValidator, TypeValidationError
 from .exceptions import InvalidSpecification
 from .utils import flaskify_endpoint, produces_json
 
 logger = logging.getLogger('connexion.operation')
 
 
-class Operation:
+class SecureOperation(object):
+    def __init__(self, security, security_definitions):
+        """
+        :param security: list of security rules the application uses by default
+        :type security: list
+        :param security_definitions: `Security Definitions Object
+            <https://github.com/swagger-api/swagger-spec/blob/master/versions/2.0.md#security-definitions-object>`_
+        :type security_definitions: dict
+        """
+        self.security = security
+        self.security_definitions = security_definitions
+
+    @property
+    def security_decorator(self):
+        """
+        Gets the security decorator for operation
+
+        From Swagger Specification:
+
+        **Security Definitions Object**
+
+        A declaration of the security schemes available to be used in the specification.
+
+        This does not enforce the security schemes on the operations and only serves to provide the relevant details
+        for each scheme.
+
+
+        **Security Requirement Object**
+
+        Lists the required security schemes to execute this operation. The object can have multiple security schemes
+        declared in it which are all required (that is, there is a logical AND between the schemes).
+
+        The name used for each property **MUST** correspond to a security scheme declared in the Security Definitions.
+
+        :rtype: types.FunctionType
+        """
+        logger.debug('... Security: %s', self.security, extra=vars(self))
+        if self.security:
+            if len(self.security) > 1:
+                logger.warning("... More than one security requirement defined. **IGNORING SECURITY REQUIREMENTS**",
+                               extra=vars(self))
+                return security_passthrough
+
+            security = self.security[0]  # type: dict
+            # the following line gets the first (and because of the previous condition only) scheme and scopes
+            # from the operation's security requirements
+
+            scheme_name, scopes = next(iter(security.items()))  # type: str, list
+            security_definition = self.security_definitions[scheme_name]
+            if security_definition['type'] == 'oauth2':
+                token_info_url = get_tokeninfo_url(security_definition)
+                if token_info_url:
+                    scopes = set(scopes)  # convert scopes to set because this is needed for verify_oauth
+                    return functools.partial(verify_oauth, token_info_url, scopes)
+                else:
+                    logger.warning("... OAuth2 token info URL missing. **IGNORING SECURITY REQUIREMENTS**",
+                                   extra=vars(self))
+            elif security_definition['type'] in ('apiKey', 'basic'):
+                logger.debug(
+                    "... Security type '%s' not natively supported by Connexion; you should handle it yourself",
+                    security_definition['type'], extra=vars(self))
+
+        # if we don't know how to handle the security or it's not defined we will usa a passthrough decorator
+        return security_passthrough
+
+
+class Operation(SecureOperation):
     """
     A single API operation on a path.
     """
 
-    def __init__(self, method, path, operation, app_produces, app_security,
-                 security_definitions, definitions, parameter_definitions, resolver, validate_responses=False):
+    def __init__(self, method, path, operation, resolver, app_produces,
+                 path_parameters=None, app_security=None, security_definitions=None,
+                 definitions=None, parameter_definitions=None, response_definitions=None,
+                 validate_responses=False):
         """
         This class uses the OperationID identify the module and function that will handle the operation
 
@@ -49,8 +121,11 @@ class Operation:
         :type path: str
         :param operation: swagger operation object
         :type operation: dict
+        :param resolver: Callable that maps operationID to a function
         :param app_produces: list of content types the application can return by default
         :type app_produces: list
+        :param path_parameters: Parameters defined in the path level
+        :type path_parameters: list
         :param app_security: list of security rules the application uses by default
         :type app_security: list
         :param security_definitions: `Security Definitions Object
@@ -59,55 +134,135 @@ class Operation:
         :param definitions: `Definitions Object
             <https://github.com/swagger-api/swagger-spec/blob/master/versions/2.0.md#definitionsObject>`_
         :type definitions: dict
-        :param resolver: Callable that maps operationID to a function
+        :param parameter_definitions: Global parameter definitions
+        :type parameter_definitions: dict
+        :param response_definitions: Global response definitions
+        :type response_definitions: dict
         :param validate_responses: True enables validation. Validation errors generate HTTP 500 responses.
         :type validate_responses: bool
         """
 
         self.method = method
         self.path = path
-        self.security_definitions = security_definitions
-        self.definitions = definitions
-        self.parameter_definitions = parameter_definitions
+        self.security_definitions = security_definitions or {}
+        self.definitions = definitions or {}
+        self.parameter_definitions = parameter_definitions or {}
+        self.response_definitions = response_definitions or {}
         self.definitions_map = {
             'definitions': self.definitions,
-            'parameters': self.parameter_definitions
+            'parameters': self.parameter_definitions,
+            'responses': self.response_definitions
         }
         self.validate_responses = validate_responses
-
         self.operation = operation
-        self.operation_id = operation['operationId']
+
         # todo support definition references
         # todo support references to application level parameters
         self.parameters = list(self.resolve_parameters(operation.get('parameters', [])))
-        self.produces = operation.get('produces', app_produces)
-        self.endpoint_name = flaskify_endpoint(self.operation_id)
+        if path_parameters:
+            self.parameters += list(self.resolve_parameters(path_parameters))
+
         self.security = operation.get('security', app_security)
-        self.__undecorated_function = resolver(self.operation_id)
+        self.produces = operation.get('produces', app_produces)
+
+        resolution = resolver.resolve(self)
+        self.operation_id = resolution.operation_id
+        self.endpoint_name = flaskify_endpoint(self.operation_id)
+        self.__undecorated_function = resolution.function
+
+        self.validate_defaults()
+
+    def validate_defaults(self):
+        for param in self.parameters:
+            try:
+                if param['in'] == 'query' and 'default' in param:
+                    validation.validate_type(param, param['default'], 'query', param['name'])
+            except (TypeValidationError, ValidationError):
+                raise InvalidSpecification('The parameter \'{param_name}\' has a default value which is not of'
+                                           ' type \'{param_type}\''.format(param_name=param['name'],
+                                                                           param_type=param['type']))
 
     def resolve_reference(self, schema):
-        schema = schema.copy()  # avoid changing the original schema
-        reference = schema.get('$ref')  # type: str
+        schema = deepcopy(schema)  # avoid changing the original schema
+        self.check_references(schema)
+
+        # find the object we need to resolve/update if this is not a proper SchemaObject
+        # e.g a response or parameter object
+        for obj in schema, schema.get('items'):
+            reference = obj and obj.get('$ref')  # type: str
+            if reference:
+                break
         if reference:
-            if not reference.startswith('#/'):
-                raise InvalidSpecification(
-                    "{method} {path}  '$ref' needs to start with '#/'".format(**vars(self)))
-            path = reference.split('/')
-            definition_type = path[1]
-            try:
-                definitions = self.definitions_map[definition_type]
-            except KeyError:
-                raise InvalidSpecification(
-                    "{method} {path}  '$ref' needs to point to definitions or parameters".format(**vars(self)))
-            definition_name = path[-1]
-            try:
-                schema.update(definitions[definition_name])
-            except KeyError:
-                raise InvalidSpecification("{method} {path} Definition '{definition_name}' not found".format(
-                    definition_name=definition_name, method=self.method, path=self.path))
-            del schema['$ref']
+            definition = deepcopy(self._retrieve_reference(reference))
+            # Update schema
+            obj.update(definition)
+            del obj['$ref']
+
+        # if there is a schema object on this param or response, then we just
+        # need to include the defs and it can be validated by jsonschema
+        if 'schema' in schema:
+            schema['schema']['definitions'] = self.definitions
+            return schema
+
         return schema
 
+    def check_references(self, schema):
+        """
+        Searches the keys and values of a schema object for json references.
+        If it finds one, it attempts to locate it and will thrown an exception
+        if the reference can't be found in the definitions dictionary.
+
+        :param schema: The schema object to check
+        :type schema: dict
+        :raises InvalidSpecification: raised when a reference isn't found
+        """
+
+        stack = [schema]
+        visited = set()
+        while stack:
+            schema = stack.pop()
+            for k, v in schema.items():
+                if k == "$ref":
+                    if v in visited:
+                        continue
+                    visited.add(v)
+                    stack.append(self._retrieve_reference(v))
+                elif isinstance(v, (list, tuple)):
+                    for item in v:
+                        if hasattr(item, "items"):
+                            stack.append(item)
+                elif hasattr(v, "items"):
+                    stack.append(v)
+
+    def _retrieve_reference(self, reference):
+        if not reference.startswith('#/'):
+            raise InvalidSpecification(
+                "{method} {path} '$ref' needs to start with '#/'".format(**vars(self)))
+        path = reference.split('/')
+        definition_type = path[1]
+        try:
+            definitions = self.definitions_map[definition_type]
+        except KeyError:
+            ref_possible = ', '.join(self.definitions_map.keys())
+            raise InvalidSpecification(
+                "{method} {path} $ref \"{reference}\" needs to point to one of: "
+                "{ref_possible}".format(
+                    method=self.method,
+                    path=self.path,
+                    reference=reference,
+                    ref_possible=ref_possible
+                ))
+        definition_name = path[-1]
+        try:
+            # Get sub definition
+            definition = deepcopy(definitions[definition_name])
+        except KeyError:
+            raise InvalidSpecification(
+                "{method} {path} Definition '{definition_name}' not found".format(
+                    definition_name=definition_name, method=self.method, path=self.path))
+
+        return definition
+
     def get_mimetype(self):
         if produces_json(self.produces):  # endpoint will return json
             try:
@@ -125,6 +280,9 @@ class Operation:
             param = self.resolve_reference(param)
             yield param
 
+    def get_path_parameter_types(self):
+        return {p['name']: p.get('type') for p in self.parameters if p['in'] == 'path'}
+
     @property
     def body_schema(self):
         """
@@ -146,9 +304,6 @@ class Operation:
 
         body_parameters = body_parameters[0] if body_parameters else {}
         schema = body_parameters.get('schema')  # type: dict
-
-        if schema:
-            schema = self.resolve_reference(schema)
         return schema
 
     @property
@@ -159,15 +314,7 @@ class Operation:
         :rtype: types.FunctionType
         """
 
-        parameters = []
-        for param in self.parameters:  # resolve references
-            param = param.copy()
-            schema = param.get('schema')
-            if schema:
-                schema = self.resolve_reference(schema)
-            param['schema'] = schema
-            parameters.append(param)
-        function = parameter_to_arg(parameters, self.__undecorated_function)
+        function = parameter_to_arg(self.parameters, self.__undecorated_function)
 
         if self.validate_responses:
             logger.debug('... Response validation enabled.')
@@ -183,7 +330,7 @@ class Operation:
             function = validation_decorator(function)
 
         # NOTE: the security decorator should be applied last to check auth before anything else :-)
-        security_decorator = self.__security_decorator
+        security_decorator = self.security_decorator
         logger.debug('... Adding security decorator (%r)', security_decorator, extra=vars(self))
         function = security_decorator(function)
 
@@ -224,62 +371,6 @@ class Operation:
         else:
             return BaseSerializer()
 
-    @property
-    def __security_decorator(self):
-        """
-        Gets the security decorator for operation
-
-        From Swagger Specification:
-
-        **Security Definitions Object**
-
-        A declaration of the security schemes available to be used in the specification.
-
-        This does not enforce the security schemes on the operations and only serves to provide the relevant details
-        for each scheme.
-
-
-        **Security Requirement Object**
-
-        Lists the required security schemes to execute this operation. The object can have multiple security schemes
-        declared in it which are all required (that is, there is a logical AND between the schemes).
-
-        The name used for each property **MUST** correspond to a security scheme declared in the Security Definitions.
-
-        :rtype: types.FunctionType
-        """
-        logger.debug('... Security: %s', self.security, extra=vars(self))
-        if self.security:
-            if len(self.security) > 1:
-                logger.warning("... More than one security requirement defined. **IGNORING SECURITY REQUIREMENTS**",
-                               extra=vars(self))
-                return security_passthrough
-
-            security = self.security[0]  # type: dict
-            # the following line gets the first (and because of the previous condition only) scheme and scopes
-            # from the operation's security requirements
-
-            scheme_name, scopes = next(iter(security.items()))  # type: str, list
-            security_definition = self.security_definitions[scheme_name]
-            if security_definition['type'] == 'oauth2':
-                token_info_url = security_definition.get('x-tokenInfoUrl', os.getenv('HTTP_TOKENINFO_URL'))
-                if token_info_url:
-                    scopes = set(scopes)  # convert scopes to set because this is needed for verify_oauth
-                    return functools.partial(verify_oauth, token_info_url, scopes)
-                else:
-                    logger.warning("... OAuth2 token info URL missing. **IGNORING SECURITY REQUIREMENTS**",
-                                   extra=vars(self))
-            elif security_definition['type'] in ('apiKey', 'basic'):
-                logger.debug(
-                    "... Security type '%s' not natively supported by Connexion; you should handle it yourself",
-                    security_definition['type'], extra=vars(self))
-            else:
-                logger.warning("... Security type '%s' unknown. **IGNORING SECURITY REQUIREMENTS**",
-                               security_definition['type'], extra=vars(self))
-
-        # if we don't know how to handle the security or it's not defined we will usa a passthrough decorator
-        return security_passthrough
-
     @property
     def __validation_decorators(self):
         """
